/* rpminfo.c
 *
 * librpminfo source code
 *
 * This library contains source code obtained from the RPM library (see
 * http://rpm.org/) which is licensed under the terms of the GNU Library
 * General Public License (version 2, June 1991).  It therefore contains
 * material whose copyright is owned by the developers of the RPM library.
 *
 * Modifications and additions made by David S. Peterson of BigFix, Inc.
 * <dave_peterson@bigfix.com>.
 * Modifications and additions Copyright (C) 2009 BigFix, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License, version 2 as
 * published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License, version 2 for more details.
 *
 * You should have received a copy of the GNU Library General Public License,
 * version 2 along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "rpminfo.h"
#include "export_sym.h"
#include <string.h>
#include <ctype.h>
#include <stddef.h>

struct rpminfo_context
 { /* externally specified functions for allocating/freeing memory */
   rpminfo_mem_alloc_context_t mem_ctx;

   /* eventually other stuff will go here */
 };

EXPORT_SYM rpminfo_context_t* rpminfo_alloc_context
      (const rpminfo_mem_alloc_context_t* c)
 { rpminfo_context_t* ret;

   ret = c->alloc_fn(sizeof(*ret));

   if (ret == NULL)
      return NULL;

   ret->mem_ctx = *c;
   return ret;
 }

EXPORT_SYM void rpminfo_free_context(rpminfo_context_t* ctx)
 { rpminfo_mem_free_fn_t free_fn;

   free_fn = ctx->mem_ctx.free_fn;
   free_fn(ctx);
 }

/* Similar to strcmp() except strings are represented a bit differently.  's1'
 * points to first character of first string and 's1end' points one character
 * past last character of first string (where terminating null character would
 * normally be).  Likewise, 's2' points to first character of second string and
 * 's2end' points one character past last character of second string.  Return 0
 * if strings are identical.  Return a value < 0 if first string is less than
 * second string.  Return a value > 0 if first string is greater than second
 * string.
 */
static int strcompare(const char *s1, const char *s1end, const char *s2,
                      const char *s2end)
 { ptrdiff_t s1len, s2len, remaining;
   const char *p1, *p2;
   size_t n;
   int result;

   s1len = s1end - s1;
   s2len = s2end - s2;
   remaining = (s1len < s2len) ? s1len : s2len;
   p1 = s1;
   p2 = s2;

   while (remaining)
    { n = (remaining > SIZE_MAX) ? SIZE_MAX : remaining;
      result = strncmp(p1, p2, n);

      if (result)
         return result;

      remaining -= n;
      p1 += n;
      p2 += n;
    }

   if (s1len == s2len)
      return 0;

   return (s1len < s2len) ? -1 : 1;
 }

EXPORT_SYM int rpminfo_vercmp(rpminfo_context_t* ctx, const char *a,
                              const char *b)
{
    const char *str1 = a, *str2 = b;
    const char * one, * two;
    int rc;
    int isnum;

    /* easy comparison to see if versions are identical */
    if (!strcmp(a, b)) return 0;

    one = str1;
    two = str2;

    /* loop through each version segment of str1 and str2 and compare them */
    while (*one && *two) {
        while (*one && !isalnum(*one)) one++;
        while (*two && !isalnum(*two)) two++;

        /* If we ran to the end of either, we are finished with the loop */
        if (!(*one && *two)) break;

        str1 = one;
        str2 = two;

        /* grab first completely alpha or completely numeric segment */
        /* leave one and two pointing to the start of the alpha or numeric */
        /* segment and walk str1 and str2 to end of segment */
        if (isdigit(*str1)) {
            while (*str1 && isdigit(*str1)) str1++;
            while (*str2 && isdigit(*str2)) str2++;
            isnum = 1;
        } else {
            while (*str1 && isalpha(*str1)) str1++;
            while (*str2 && isalpha(*str2)) str2++;
            isnum = 0;
        }

        /* this cannot happen, as we previously tested to make sure that */
        /* the first string has a non-null segment */
        if (one == str1) return -1;     /* arbitrary */

        /* take care of the case where the two version segments are */
        /* different types: one numeric, the other alpha (i.e. empty) */
        /* numeric segments are always newer than alpha segments */
        /* XXX See patch #60884 (and details) from bugzilla #50977. */
        if (two == str2) return (isnum ? 1 : -1);

        if (isnum) {
            /* this used to be done by converting the digit segments */
            /* to ints using atoi() - it's changed because long  */
            /* digit segments can overflow an int - this should fix that. */

            /* throw away any leading zeros - it's a number, right? */
            while (*one == '0') one++;
            while (*two == '0') two++;

            /* whichever number has more digits wins */
            if ((str1 - one) > (str2 - two)) return 1;
            if ((str2 - two) > (str1 - one)) return -1;
        }

        /* strcmp will return which one is greater - even if the two */
        /* segments are alpha or if they are numeric.  don't return  */
        /* if they are equal because there might be more segments to */
        /* compare */
        rc = strcompare(one, str1, two, str2);
        if (rc) return (rc < 1 ? -1 : 1);

        one = str1;
        two = str2;
    }

    /* this catches the case where all numeric and alpha segments have */
    /* compared identically but the segment sepparating characters were */
    /* different */
    if ((!*one) && (!*two)) return 0;

    /* whichever version still has characters left over wins */
    if (!*one) return -1; else return 1;
}

EXPORT_SYM int rpminfo_version_compare
      (rpminfo_context_t* ctx, const rpminfo_pkg_version_info_t* a,
       const rpminfo_pkg_version_info_t* b)
 { int result;

   if (a->epoch < b->epoch)
      return -1;

   if (a->epoch > b->epoch)
      return 1;

   result = rpminfo_vercmp(ctx, a->version, b->version);


   if (result)
      return result;

   return rpminfo_vercmp(ctx, a->release, b->release);
 }
