/* rpminfo.h
 *
 * librpminfo source code
 *
 * This library contains source code obtained from the RPM library (see
 * http://rpm.org/) which is licensed under the terms of the GNU Library
 * General Public License (version 2, June 1991).  It therefore contains
 * material whose copyright is owned by the developers of the RPM library.
 *
 * Modifications and additions made by David S. Peterson of BigFix, Inc.
 * <dave_peterson@bigfix.com>.
 * Modifications and additions Copyright (C) 2009 BigFix, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License, version 2 as
 * published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License, version 2 for more details.
 *
 * You should have received a copy of the GNU Library General Public License,
 * version 2 along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef RPM_INFO_H
#define RPM_INFO_H

#include <stdint.h>
#include <stddef.h>

#ifdef __cplusplus
extern "C"
 {
#endif

// memory allocation/deallocation function pointer types
typedef void* (*rpminfo_mem_alloc_fn_t)(size_t size);
typedef void (*rpminfo_mem_free_fn_t)(void* ptr);

// caller-specified functions for library to use when it needs to allocate/free
// memory
typedef struct rpminfo_mem_alloc_context
 { rpminfo_mem_alloc_fn_t alloc_fn;
   rpminfo_mem_free_fn_t free_fn;
 } rpminfo_mem_alloc_context_t;

// Contextual info (such as caller-specified memory allocation/deallocation
// functions) needed by rpminfo library.
struct rpminfo_context;
typedef struct rpminfo_context rpminfo_context_t;

// epoch/version/release info for an RPM package
typedef struct rpminfo_pkg_version_info
 { int32_t epoch;
   const char* version;
   const char* release;
 } rpminfo_pkg_version_info_t;

// Create a rpminfo_context_t object needed by other rpminfo library calls.
// Object will be allocated by calling alloc_fn specified by parameter 'c'.
// Allocation context specified by 'c' will then become associated with
// rpminfo_context_t being created (so library will know how to do future
// memory allocations/deallocations).  Return pointer to newly allocated
// rpminfo_context_t object, or NULL on allocation failure.
rpminfo_context_t* rpminfo_alloc_context(const rpminfo_mem_alloc_context_t* c);

// Free an rpminfo_context_t that was previously created with a call to
// rpminfo_alloc_context().  free_fn specified by rpminfo_mem_alloc_context
// associated with 'ctx' will be used to free 'ctx'.
void rpminfo_free_context(rpminfo_context_t* ctx);

/* This is a modified version of rpmvercmp() from lib/rpmvercmp.c in the RPM
 * Library version 4.6.0 source code.
 */
int rpminfo_vercmp(rpminfo_context_t* ctx, const char* a, const char* b);

/* This behaves identically to rpmVersionCompare() from lib/psm.c in the RPM
 * Library version 4.6.0 source code.  For 'a' and 'b', the epoch should be
 * given a value of 0 if it does not exist.
 */
int rpminfo_version_compare(rpminfo_context_t* ctx,
                            const rpminfo_pkg_version_info_t* a,
                            const rpminfo_pkg_version_info_t* b);

#ifdef __cplusplus
 }  // extern "C"
#endif

#endif  /* RPM_INFO_H */
